@@ -28,7 +28,7 @@ class Widget:
     :meth:`~import_export.widgets.Widget.clean` and
     :meth:`~import_export.widgets.Widget.render`.
     """
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         """
         Returns an appropriate Python object for an imported value.
 
@@ -71,7 +71,7 @@ class FloatWidget(NumberWidget):
     Widget for converting floats fields.
     """
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if self.is_empty(value):
             return None
         return float(value)
@@ -82,7 +82,7 @@ class IntegerWidget(NumberWidget):
     Widget for converting integer fields.
     """
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if self.is_empty(value):
             return None
         return int(Decimal(value))
@@ -93,7 +93,7 @@ class DecimalWidget(NumberWidget):
     Widget for converting decimal fields.
     """
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if self.is_empty(value):
             return None
         return Decimal(force_str(value))
@@ -103,9 +103,7 @@ class CharWidget(Widget):
     """
     Widget for converting text fields.
     """
-
-    def render(self, value, obj=None):
-        return force_str(value)
+    pass
 
 
 class BooleanWidget(Widget):
@@ -152,7 +150,7 @@ class BooleanWidget(Widget):
             return ""
         return self.TRUE_VALUES[0] if value else self.FALSE_VALUES[0]
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if value in self.NULL_VALUES:
             return None
         return True if value in self.TRUE_VALUES else False
@@ -175,7 +173,7 @@ class DateWidget(Widget):
             formats = (format,)
         self.formats = formats
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if not value:
             return None
         if isinstance(value, date):
@@ -211,22 +209,22 @@ class DateTimeWidget(Widget):
             formats = (format,)
         self.formats = formats
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
+        dt = None
         if not value:
             return None
         if isinstance(value, datetime):
-            return value
-        for format in self.formats:
-            try:
-                dt = datetime.strptime(value, format)
-                if settings.USE_TZ:
-                    # make datetime timezone aware so we don't compare
-                    # naive datetime to an aware one
-                    dt = timezone.make_aware(dt,
-                                             timezone.get_default_timezone())
-                return dt
-            except (ValueError, TypeError):
-                continue
+            dt = value
+        else:
+            for format_ in self.formats:
+                try:
+                    dt = datetime.strptime(value, format_)
+                except (ValueError, TypeError):
+                    continue
+        if dt:
+            if settings.USE_TZ and timezone.is_naive(dt):
+                dt = timezone.make_aware(dt)
+            return dt
         raise ValueError("Enter a valid date/time.")
 
     def render(self, value, obj=None):
@@ -254,7 +252,7 @@ class TimeWidget(Widget):
             formats = (format,)
         self.formats = formats
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if not value:
             return None
         if isinstance(value, time):
@@ -277,7 +275,7 @@ class DurationWidget(Widget):
     Widget for converting time duration fields.
     """
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if not value:
             return None
 
@@ -305,7 +303,7 @@ class SimpleArrayWidget(Widget):
         self.separator = separator
         super().__init__()
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         return value.split(self.separator) if value else []
 
     def render(self, value, obj=None):
@@ -321,7 +319,7 @@ class JSONWidget(Widget):
     tries to use single quotes and then convert it to proper JSON.
     """
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         val = super().clean(value)
         if val:
             try:
@@ -337,7 +335,8 @@ class JSONWidget(Widget):
 class ForeignKeyWidget(Widget):
     """
     Widget for a ``ForeignKey`` field which looks up a related model using
-    "natural keys" in both export and import.
+    either the PK or a user specified field that uniquely identifies the
+    instance in both export and import.
 
     The lookup field defaults to using the primary key (``pk``) as lookup
     criterion but can be customised to use any field on the related model.
@@ -370,12 +369,16 @@ class ForeignKeyWidget(Widget):
                 fields = ('author',)
 
     :param model: The Model the ForeignKey refers to (required).
-    :param field: A field on the related model used for looking up a particular object.
+    :param field: A field on the related model used for looking up a particular
+        object.
+    :param use_natural_foreign_keys: Use natural key functions to identify
+        related object, default to False
     """
-    def __init__(self, model, field='pk', *args, **kwargs):
+    def __init__(self, model, field='pk', use_natural_foreign_keys=False, **kwargs):
         self.model = model
         self.field = field
-        super().__init__(*args, **kwargs)
+        self.use_natural_foreign_keys = use_natural_foreign_keys
+        super().__init__(**kwargs)
 
     def get_queryset(self, value, row, *args, **kwargs):
         """
@@ -400,10 +403,15 @@ class ForeignKeyWidget(Widget):
         """
         return self.model.objects.all()
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         val = super().clean(value)
         if val:
-            return self.get_queryset(value, row, *args, **kwargs).get(**{self.field: val})
+            if self.use_natural_foreign_keys:
+                # natural keys will always be a tuple, which ends up as a json list.
+                value = json.loads(value)
+                return self.model.objects.get_by_natural_key(*value)
+            else:
+                return self.get_queryset(value, row, **kwargs).get(**{self.field: val})
         else:
             return None
 
@@ -414,7 +422,11 @@ class ForeignKeyWidget(Widget):
         attrs = self.field.split('__')
         for attr in attrs:
             try:
-                value = getattr(value, attr, None)
+                if self.use_natural_foreign_keys:
+                    # inbound natural keys must be a json list.
+                    return json.dumps(value.natural_key())
+                else:
+                    value = getattr(value, attr, None)
             except (ValueError, ObjectDoesNotExist):
                 # needs to have a primary key value before a many-to-many
                 # relationship can be used.
@@ -435,7 +447,7 @@ class ManyToManyWidget(Widget):
     :param field: A field on the related model. Default is ``pk``.
     """
 
-    def __init__(self, model, separator=None, field=None, *args, **kwargs):
+    def __init__(self, model, separator=None, field=None, **kwargs):
         if separator is None:
             separator = ','
         if field is None:
@@ -443,9 +455,9 @@ class ManyToManyWidget(Widget):
         self.model = model
         self.separator = separator
         self.field = field
-        super().__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
-    def clean(self, value, row=None, *args, **kwargs):
+    def clean(self, value, row=None, **kwargs):
         if not value:
             return self.model.objects.none()
         if isinstance(value, (float, int)):
