@@ -58,13 +58,14 @@ if os.environ.get('IMPORT_EXPORT_TEST_TYPE') == 'mysql-innodb':
         'default': {
             'ENGINE': 'django.db.backends.mysql',
             'NAME': 'import_export',
-            'USER': os.environ.get('IMPORT_EXPORT_MYSQL_USER', 'root'),
-            'PASSWORD': os.environ.get('IMPORT_EXPORT_MYSQL_PASSWORD', 'password'),
+            'USER': os.environ.get('IMPORT_EXPORT_MYSQL_USER'),
+            'PASSWORD': os.environ.get('IMPORT_EXPORT_MYSQL_PASSWORD'),
             'HOST': '127.0.0.1',
             'PORT': 3306,
             'TEST': {
                 'CHARSET': 'utf8',
                 'COLLATION': 'utf8_general_ci',
+                'NAME': 'test_import_export'
             }
         }
     }
@@ -108,3 +109,5 @@ LOGGING = {
 
 
 USE_TZ = False
+if django.VERSION >= (4, 1):
+    FORM_RENDERER = "django.forms.renderers.DjangoDivFormRenderer"
